@@ -41,15 +41,19 @@ run ("-w":rest)                 = run rest
 run (('-':'l':xs):rest)         = run (('-':xs):rest)
 run (('-':'w':xs):rest)         = run (('-':xs):rest)
 run (('-':'d':xs):rest)         = run (('-':xs):rest)
-run (('-':'e':prog@(_:_)):args) = doRun "-" args prog
-run ("-e":prog:args)            = doRun "-" args prog
-run ("-h":_)                    = printHelp
+run (('-':'e':prog@(_:_)):args) = doRun "-e" args prog
+run ("-e":prog:args)            = doRun "-e" args prog
+run ("-h":_)                    = printCommandLineHelp
+run ("--help":_)                = printCommandLineHelp
 run ("-V":_)                    = printConfigInfo
 run ("-v":_)                    = banner
 run ("--version":_)             = banner
-run ("-c":"-e":prog:_)          = doParse prog
-run ("-ce":prog:_)              = doParse prog
-run ("-c":file:_)               = readFile file >>= doParse
+run ("-c":"-e":prog:_)          = doCheck "-e" prog
+run ("-ce":prog:_)              = doCheck "-e" prog
+run ("-c":file:_)               = readFile file >>= doCheck file
+run ("-C":"-e":prog:_)          = doDump "-e" prog
+run ("-Ce":prog:_)              = doDump "-e" prog
+run ("-C":file:_)               = readFile file >>= doDump file
 run ("-":_)                     = do
     prog <- getContents
     doRun "-" [] prog
@@ -61,28 +65,49 @@ run []                          = do
         else run ["-"]
 
 repLoop :: IO ()
-repLoop
-   = do command <- getCommand
-        case command of
-           CmdQuit      -> putStrLn "Leaving pugs."
-           CmdLoad fn   -> load fn
-           CmdEval prog -> doEval [] prog >> repLoop
-           CmdParse prog-> doParse prog >> repLoop
-           CmdHelp      -> printHelp >> repLoop
-           _            -> internalError "repLoop unimplemented command: "
+repLoop = do
+    command <- getCommand
+    case command of
+        CmdQuit             -> putStrLn "Leaving pugs."
+        CmdLoad fn          -> load fn
+        CmdEval trace prog  -> doEval trace [] prog >> repLoop
+        CmdParse prog       -> doParse "<interactive>" prog >> repLoop
+        CmdHelp             -> printInteractiveHelp >> repLoop
+        _                   -> internalError "unimplemented command"
 
 load _ = return ()
 
-parse = doParse
-eval prog = doEval [] prog
+parse = doParse "<interactive>"
+eval prog = doEval True [] prog
 
-doParse prog = do
+doCheck = doParseWith $ \_ name -> do
+    putStrLn $ name ++ " syntax OK"
+
+doDump = doParseWith $ \exp _ -> do
+    fh <- openFile "dump.ast" WriteMode
+    hPutStrLn fh $ show exp
+    hClose fh
+
+doParseWith f name prog = do
     env <- emptyEnv []
-    runRule env (putStrLn . pretty) ruleProgram "<interactive>" prog
+    runRule env (f' . envBody) ruleProgram name $ decodeUTF8 prog
+    where
+    f' (Val err@(VError _ _)) = do
+        hPutStrLn stderr $ pretty err
+        exitFailure
+    f' exp = f exp name
+    
 
-doEval :: [String] -> String -> IO ()
-doEval = do
-    runProgramWith id (putStrLn . pretty) "<interactive>"
+doParse name prog = do
+    env <- emptyEnv []
+    runRule env (putStrLn . pretty) ruleProgram name $ decodeUTF8 prog
+
+doEval :: Bool -> [String] -> String -> IO ()
+doEval trace = do
+    runProgramWith f (putStrLn . pretty) "<interactive>"
+    where
+    f | trace       = id
+      | otherwise   = \e -> e{ envDebug = Nothing }
 
 doRun :: String -> [String] -> String -> IO ()
 doRun = do
@@ -98,20 +123,38 @@ runFile :: String -> IO ()
 runFile file = do
     withArgs [file] main
 
-runProgramWith :: (Env -> Env) -> (Val -> IO ()) -> VStr -> [VStr] -> String -> IO ()
+runProgramWith :: 
+    (Env -> Env) -> (Val -> IO ()) -> VStr -> [VStr] -> String -> IO ()
 runProgramWith fenv f name args prog = do
+    env <- prepareEnv name args
+    val <- runEnv $ runRule (fenv env) id ruleProgram name $ decodeUTF8 prog
+    f val
+
+runEnv env = do
+    (`runReaderT` env) $ do
+        (`runContT` return) $ resetT $ do
+            evaluateMain (envBody env)
+
+runAST ast = do
+    hSetBuffering stdout NoBuffering 
+    name <- getProgName
+    args <- getArgs
+    env  <- prepareEnv name args
+    runEnv env{ envBody = ast, envDebug = Nothing }
+
+prepareEnv name args = do
     environ <- getEnvironment
     let envFM = listToFM $ [ (VStr k, VStr v) | (k, v) <- environ ]
-        p6lib = maybeToList $ lookup "PERL6LIB" environ
-    p5libs  <- mapM fixLib $ catMaybes [lookup "PERL5LIB" environ, lookup "PERLLIB" environ]
+    libs    <- getLibs environ
     progSV  <- newMVal $ VStr name
     endAV   <- newMVal $ VList []
-    incAV   <- newMVal $ VList (map VStr $ p6lib ++ concat p5libs ++ incs)
+    incAV   <- newMVal $ VList (map VStr libs)
     argsAV  <- newMVal $ VList (map VStr args)
     inGV    <- newMVal $ VHandle stdin
     outGV   <- newMVal $ VHandle stdout
     errGV   <- newMVal $ VHandle stderr
-    env <- emptyEnv
+    errSV   <- newMVal $ VStr ""
+    emptyEnv
         [ Symbol SGlobal "@*ARGS"       $ Val argsAV
         , Symbol SGlobal "@*INC"        $ Val incAV
         , Symbol SGlobal "$*PROGNAME"   $ Val progSV
@@ -119,50 +162,38 @@ runProgramWith fenv f name args prog = do
         , Symbol SGlobal "$*IN"         $ Val inGV
         , Symbol SGlobal "$*OUT"        $ Val outGV
         , Symbol SGlobal "$*ERR"        $ Val errGV
+        , Symbol SGlobal "$!"           $ Val errSV
         , Symbol SGlobal "%*ENV" (Val . VHash . MkHash $ envFM)
-        , Symbol SGlobal "%=POD"        (Val . VHash . MkHash $ emptyFM) -- wrong: pkg
+        -- XXX What would this even do?
+        -- , Symbol SGlobal "%=POD"        (Val . VHash . MkHash $ emptyFM) 
         , Symbol SGlobal "@=POD"        (Val . VArray . MkArray $ [])
         , Symbol SGlobal "$=POD"        (Val . VStr $ "")
         , Symbol SGlobal "$?OS"         (Val . VStr $ config_osname)
         ]
---    str <- return "" -- getContents
-    let env' = runRule (fenv env) id ruleProgram name prog
-    val <- (`runReaderT` env') $ do
-        (`runContT` return) $ resetT $ do
-            evaluateMain (envBody env')
-    f val
-    where
-    fixLib str = do
-        let path = str ++ "/Perl6/lib"
-        exists <- doesDirectoryExist path
-        return $ if exists then [path] else []
 
-incs = [ "./lib/Perl6/lib"
-       , "../lib/Perl6/lib"
-       , "../../lib/Perl6/lib"
-       , config_archlib
-       , config_privlib
-       , config_sitearch
-       , config_sitelib
-       , "."
-       ]
+getLibs :: [(String, String)] -> IO [String]
+getLibs environ = return $ filter (not . null) libs
+    where
+    envlibs nm = maybe [] (split config_path_sep) $ nm `lookup` environ
+    libs =  envlibs "PERL6LIB"
+         ++ [ config_archlib
+            , config_privlib
+            , config_sitearch
+            , config_sitelib
+            ]
+         ++ [ "." ]
 
 printConfigInfo :: IO ()
-printConfigInfo = putStrLn $ unlines $
-    [ "@*INC:" ] ++ incs
-
-{-
-main = do
-    -- (optsIO, rest, errs) <- return . getOpt Permute options $ procArgs args
-
-options :: [OptDescr (Opts -> Opts)]
-options =
-    [ reqArg "e" ["eval"]           "command"       "Command-line program"
-        (\s o -> o { encodings          = split "," s })
-    , noArg  "d" ["debug"]                          "Turn on debugging"
-        (\s o -> o { inputFile          = s })
-    , noArg  "h" ["help"]                           "Show help"
-        (\o   -> o { showHelp           = usage "" })
-    ]
--}
-
+printConfigInfo = do
+    environ <- getEnvironment
+    libs <- getLibs environ
+    putStrLn $ unlines $
+        ["Summary of pugs configuration:"
+        ,""
+        ,"archlib: " ++ config_archlib
+        ,"privlib: " ++ config_privlib
+        ,"sitearch: " ++ config_sitearch
+        ,"sitelib: " ++ config_sitelib
+        ,""
+        ] ++
+        [ "@*INC:" ] ++ libs
